/*
 * This file is part of Amtk - Actions, Menus and Toolbars Kit
 *
 * Copyright 2017, 2018 - Sébastien Wilmet <swilmet@gnome.org>
 *
 * Amtk is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * Amtk is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef AMTK_H
#define AMTK_H

#define AMTK_H_INSIDE

#include <amtk/amtk-types.h>
#include <amtk/amtk-enum-types.h>

#include <amtk/amtk-action-info.h>
#include <amtk/amtk-action-info-store.h>
#include <amtk/amtk-action-info-central-store.h>
#include <amtk/amtk-action-map.h>
#include <amtk/amtk-application-window.h>
#include <amtk/amtk-factory.h>
#include <amtk/amtk-gmenu.h>
#include <amtk/amtk-init.h>
#include <amtk/amtk-menu-item.h>
#include <amtk/amtk-menu-shell.h>
#include <amtk/amtk-shortcuts.h>
#include <amtk/amtk-utils.h>

#undef AMTK_H_INSIDE

#endif /* AMTK_H */
